import os
import sys
import unittest
import subprocess
import time
import urllib

sys.path.insert(0, os.path.join(os.path.dirname(os.path.abspath(__file__)), '..'))

TEST_DEFAULTS = {
  'ROOT_URLCONF': 'jsontesturls',
  'DEBUG': True,
  'DEBUG_PROPAGATE_EXCEPTIONS': True,
  'DATETIME_FORMAT': 'N j, Y, P',
  'USE_I18N': False,
  'INSTALLED_APPS': (
    'jsonrpc',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions'),
  'DATABASE_ENGINE': 'sqlite3',
  'DATABASE_NAME': 'test.sqlite3',
  'MIDDLEWARE_CLASSES': (
      'django.middleware.common.CommonMiddleware',
      'django.contrib.sessions.middleware.SessionMiddleware',
      'django.middleware.csrf.CsrfViewMiddleware',
      'django.contrib.auth.middleware.AuthenticationMiddleware',
  ),
  'AUTHENTICATION_BACKENDS': ('django.contrib.auth.backends.ModelBackend',),
  'TEMPLATE_LOADERS': (
      'django.template.loaders.filesystem.load_template_source',
      'django.template.loaders.app_directories.load_template_source'),
}

from django.conf import settings
settings.configure(**TEST_DEFAULTS)

from django.core import management
from django.contrib.auth.models import User
from jsonrpc import jsonrpc_method, _parse_sig, Any, SortedDict
from jsonrpc.proxy import ServiceProxy
from jsonrpc._json import loads, dumps
from jsonrpc.site import validate_params
from jsonrpc.exceptions import *
from jsonrpc.types import *


def _call(host, req):
  return loads(urllib.urlopen(host, dumps(req)).read())


def json_serve_thread():
  from wsgiref.simple_server import make_server
  from django.core.handlers.wsgi import WSGIHandler
  http = make_server('', 8999, WSGIHandler())
  http.serve_forever()

@jsonrpc_method('jsonrpc.test')
def echo(request, string):
  """Returns whatever you give it."""
  return string

@jsonrpc_method('jsonrpc.testAuth', authenticated=True)
def echoAuth(requet, string):
  return string

@jsonrpc_method('jsonrpc.notify')
def notify(request, string):
  pass

@jsonrpc_method('jsonrpc.fails')
def fails(request, string):
  raise IndexError

@jsonrpc_method('jsonrpc.strangeEcho')
def strangeEcho(request, string, omg, wtf, nowai, yeswai='Default'):
  return [string, omg, wtf, nowai, yeswai]

@jsonrpc_method('jsonrpc.safeEcho', safe=True)
def safeEcho(request, string):
  return string

@jsonrpc_method('jsonrpc.strangeSafeEcho', safe=True)
def strangeSafeEcho(request, *args, **kwargs):
  return strangeEcho(request, *args, **kwargs)

@jsonrpc_method('jsonrpc.checkedEcho(string=str, string2=str) -> str', safe=True, validate=True)
def protectedEcho(request, string, string2):
  return string + string2

@jsonrpc_method('jsonrpc.checkedArgsEcho(string=str, string2=str)', validate=True)
def protectedArgsEcho(request, string, string2):
  return string + string2

@jsonrpc_method('jsonrpc.checkedReturnEcho() -> String', validate=True)
def protectedReturnEcho(request, string, string2):
  return string + string2

@jsonrpc_method('jsonrpc.authCheckedEcho(Object, Array) -> Object', validate=True)
def authCheckedEcho(request, obj1, arr1):
  return {'obj1': obj1, 'arr1': arr1}

@jsonrpc_method('jsonrpc.varArgs(String, String, str3=String) -> Array', validate=True)
def checkedVarArgsEcho(request, *args, **kw):
  return list(args) + kw.values()


class JSONRPCFunctionalTests(unittest.TestCase):
  def test_method_parser(self):
    working_sigs = [
      ('jsonrpc', 'jsonrpc', SortedDict(), Any),
      ('jsonrpc.methodName', 'jsonrpc.methodName', SortedDict(), Any),
      ('jsonrpc.methodName() -> list', 'jsonrpc.methodName', SortedDict(), list),
      ('jsonrpc.methodName(str, str, str ) ', 'jsonrpc.methodName', SortedDict([('a', str), ('b', str), ('c', str)]), Any),
      ('jsonrpc.methodName(str, b=str, c=str)', 'jsonrpc.methodName', SortedDict([('a', str), ('b', str), ('c', str)]), Any),
      ('jsonrpc.methodName(str, b=str) -> dict', 'jsonrpc.methodName', SortedDict([('a', str), ('b', str)]), dict),
      ('jsonrpc.methodName(str, str, c=Any) -> Any', 'jsonrpc.methodName', SortedDict([('a', str), ('b', str), ('c', Any)]), Any),
      ('jsonrpc(Any ) ->  Any', 'jsonrpc', SortedDict([('a', Any)]), Any),
    ]
    error_sigs = [
      ('jsonrpc(str) -> nowai', ValueError),
      ('jsonrpc(nowai) -> Any', ValueError),
      ('jsonrpc(nowai=str, str)', ValueError),
      ('jsonrpc.methodName(nowai*str) -> Any', ValueError)
    ]
    for sig in working_sigs:
      ret = _parse_sig(sig[0], list(iter(sig[2])))
      self.assertEquals(ret[0], sig[1])
      self.assertEquals(ret[1], sig[2])
      self.assertEquals(ret[2], sig[3])
    for sig in error_sigs:
      e = None
      try:
        _parse_sig(sig[0], ['a'])
      except Exception, exc:
        e = exc
      self.assert_(type(e) is sig[1])
  
  def test_validate_args(self):
    sig = 'jsonrpc(String, String) -> String'
    M = jsonrpc_method(sig, validate=True)(lambda r, s1, s2: s1+s2)
    self.assert_(validate_params(M, {'params': ['omg', u'wtf']}) is None)
    
    E = None
    try:
      validate_params(M, {'params': [['omg'], ['wtf']]})
    except Exception, e:
      E = e
    self.assert_(type(E) is InvalidParamsError)
  
  def test_validate_args_any(self):
    sig = 'jsonrpc(s1=Any, s2=Any)'
    M = jsonrpc_method(sig, validate=True)(lambda r, s1, s2: s1+s2)
    self.assert_(validate_params(M, {'params': ['omg', 'wtf']}) is None)
    self.assert_(validate_params(M, {'params': [['omg'], ['wtf']]}) is None)
    self.assert_(validate_params(M, {'params': {'s1': 'omg', 's2': 'wtf'}}) is None)
  
  def test_types(self):
    assert type(u'') == String
    assert type('') == String
    assert not type('') == Object
    assert not type([]) == Object
    assert type([]) == Array
    assert type('') == Any
    assert Any.kind('') == String
    assert Any.decode('str') == String
    assert Any.kind({}) == Object
    assert Any.kind(None) == Nil


class ServiceProxyText(unittest.TestCase):      
  def setUp(self):
    self.proc = subprocess.Popen([sys.executable, 
      os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test.py'),
      'serve'])
    time.sleep(.5)
    self.host = 'http://127.0.0.1:8999/json/'
  
  def tearDown(self):
    self.proc.terminate()
    self.proc.wait()
  
  def test_positional_args(self):
    proxy = ServiceProxy(self.host)
    self.assert_(proxy.jsonrpc.test('Hello')[u'result'] == 'Hello')
    try:
      proxy.jsonrpc.test(string='Hello')
    except Exception, e:
      self.assert_(e.args[0] == 'Unsupport arg type for JSON-RPC 1.0 '
                                '(the default version for this client, '
                                'pass version="2.0" to use keyword arguments)')
    else:
      self.assert_(False, 'Proxy didnt warn about version mismatch')
  
  def test_keyword_args(self):        
    proxy = ServiceProxy(self.host, version='2.0')
    self.assert_(proxy.jsonrpc.test(string='Hello')[u'result'] == 'Hello')
    self.assert_(proxy.jsonrpc.test('Hello')[u'result'] == 'Hello')


class JSONRPCTest(unittest.TestCase):
  def setUp(self):
    self.proc = subprocess.Popen([sys.executable, 
      os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test.py'),
      'serve'])
    time.sleep(.5)
    self.host = 'http://127.0.0.1:8999/json/'
    self.proxy10 = ServiceProxy(self.host, version='1.0')
    self.proxy20 = ServiceProxy(self.host, version='2.0')
  
  def tearDown(self):
    self.proc.terminate()
    self.proc.wait()
  
  def test_10(self):
    self.assertEqual(
      self.proxy10.jsonrpc.test('this is a string')[u'result'], 
      u'this is a string')
  
  def test_11(self):
    req = {
      u'version': u'1.1',
      u'method': u'jsonrpc.test',
      u'params': [u'this is a string'],
      u'id': u'holy-mother-of-god'
    }
    resp = _call(self.host, req)
    self.assertEquals(resp[u'id'], req[u'id'])
    self.assertEquals(resp[u'result'], req[u'params'][0])
  
  def test_10_notify(self):
    pass
  
  def test_11_positional_mixed_args(self):
    req = {
      u'version': u'1.1',
      u'method': u'jsonrpc.strangeEcho',
      u'params': {u'1': u'this is a string', u'2': u'this is omg', 
                  u'wtf': u'pants', u'nowai': 'nopants'},
      u'id': u'toostrange'
    }
    resp = _call(self.host, req)
    self.assertEquals(resp[u'result'][-1], u'Default')
    self.assertEquals(resp[u'result'][1], u'this is omg')
    self.assertEquals(resp[u'result'][0], u'this is a string')
    self.assert_(u'error' not in resp)
  
  def test_11_GET(self):
    pass
  
  def test_11_GET_unsafe(self):
    pass
  
  def test_11_GET_mixed_args(self):
    params = {u'1': u'this is a string', u'2': u'this is omg', 
              u'wtf': u'pants', u'nowai': 'nopants'}
    url = "%s%s?%s" % (
      self.host, 'jsonrpc.strangeSafeEcho',
      (''.join(['%s=%s&' % (k, urllib.quote(v)) for k, v in params.iteritems()])).rstrip('&')
    )
    resp = loads(urllib.urlopen(url).read())
    self.assertEquals(resp[u'result'][-1], u'Default')
    self.assertEquals(resp[u'result'][1], u'this is omg')
    self.assertEquals(resp[u'result'][0], u'this is a string')
    self.assert_(u'error' not in resp)
  
  def test_20_checked(self):
    self.assertEqual(
      self.proxy10.jsonrpc.varArgs('o', 'm', 'g')[u'result'],
      ['o', 'm', 'g']
    )
    self.assert_(self.proxy10.jsonrpc.varArgs(1,2,3)[u'error'])
  
  def test_11_service_description(self):
    pass
  
  def test_20_keyword_args(self):
    self.assertEqual(
      self.proxy20.jsonrpc.test(string='this is a string')[u'result'],
      u'this is a string')
  
  def test_20_positional_args(self):
    self.assertEqual(
      self.proxy20.jsonrpc.test('this is a string')[u'result'],
      u'this is a string')
  
  def test_20_notify(self):
    req = {
      u'jsonrpc': u'2.0', 
      u'method': u'jsonrpc.notify', 
      u'params': [u'this is a string'], 
      u'id': None
    }
    resp = urllib.urlopen(self.host, dumps(req)).read()
    self.assertEquals(resp, '')
  
  def test_20_batch(self):
    req = [{
      u'jsonrpc': u'2.0',
      u'method': u'jsonrpc.test',
      u'params': [u'this is a string'],
      u'id': u'id-'+unicode(i)
    } for i in range(5)]
    resp = loads(urllib.urlopen(self.host, dumps(req)).read())
    self.assertEquals(len(resp), len(req))
    for i, D in enumerate(resp):
      self.assertEquals(D[u'result'], req[i][u'params'][0])
      self.assertEquals(D[u'id'], req[i][u'id'])
  
  def test_20_batch_with_errors(self):
    req = [{
      u'jsonrpc': u'2.0',
      u'method': u'jsonrpc.test' if not i % 2 else u'jsonrpc.fails',
      u'params': [u'this is a string'],
      u'id': u'id-'+unicode(i)
    } for i in range(10)]
    resp = loads(urllib.urlopen(self.host, dumps(req)).read())
    self.assertEquals(len(resp), len(req))
    for i, D in enumerate(resp):
      if not i % 2:
        self.assertEquals(D[u'result'], req[i][u'params'][0])
        self.assertEquals(D[u'id'], req[i][u'id'])
      else:
        self.assertEquals(D[u'result'], None)
        self.assert_(u'error' in D)
        self.assertEquals(D[u'error'][u'code'], 500)
  
  def test_authenticated_ok(self):
    self.assertEquals(
      self.proxy10.jsonrpc.testAuth(
        'sammeh', 'password', u'this is a string')[u'result'],
      u'this is a string')
  
  def test_authenticated_ok_kwargs(self):
    self.assertEquals(
      self.proxy20.jsonrpc.testAuth(
        username='sammeh', password='password', string=u'this is a string')[u'result'],
      u'this is a string')
  
  def test_authenticated_fail_kwargs(self):
    try:
      self.proxy20.jsonrpc.testAuth(
        username='osammeh', password='password', string=u'this is a string')
    except IOError, e:
      self.assertEquals(e.args[1], 401)
    else:
      self.assert_(False, 'Didnt return status code 401 on unauthorized access')
  
  def test_authenticated_fail(self):
    try:
      self.proxy10.jsonrpc.testAuth(
        'osammeh', 'password', u'this is a string')
    except IOError, e:
      self.assertEquals(e.args[1], 401)
    else:
      self.assert_(False, 'Didnt return status code 401 on unauthorized access')


if __name__ == '__main__':
  if len(sys.argv) > 1 and sys.argv[1].strip() == 'serve':
    management.call_command('syncdb', interactive=False)
    try:
      User.objects.create_user(username='sammeh', email='sam@rf.com', password='password').save()
    except:
      pass
    json_serve_thread()
  else:
    unittest.main()




from _json import dumps
from django.http import HttpResponse
from django.shortcuts import render_to_response
from jsonrpc.site import jsonrpc_site
from jsonrpc import mochikit

def browse(request):
  if (request.GET.get('f', None) == 'mochikit.js'):
    return HttpResponse(mochikit.mochikit, content_type='application/x-javascript')
  if (request.GET.get('f', None) == 'interpreter.js'):
    return HttpResponse(mochikit.interpreter, content_type='application/x-javascript')
  desc = jsonrpc_site.service_desc()
  return render_to_response('browse.html', {
    'methods': desc['procs'],
    'method_names_str': dumps(
      [m['name'] for m in desc['procs']])
  })



from functools import wraps
from uuid import uuid1
from jsonrpc._json import loads, dumps
from jsonrpc.exceptions import *
from jsonrpc.types import *
empty_dec = lambda f: f
try:
  from django.views.decorators.csrf import csrf_exempt
except (NameError, ImportError):
  csrf_exempt = empty_dec

NoneType = type(None)
encode_kw = lambda p: dict([(str(k), v) for k, v in p.iteritems()])

def encode_kw11(p):
  if not type(p) is dict:
    return {}
  ret = p.copy()
  removes = []
  for k, v in ret.iteritems():
    try:
      int(k)
    except ValueError:
      pass
    else:
      removes.append(k)
  for k in removes:
    ret.pop(k)
  return ret

def encode_arg11(p):
  if type(p) is list:
    return p
  elif not type(p) is dict:
    return []
  else:
    pos = []
    d = encode_kw(p)
    for k, v in d.iteritems():
      try:
        pos.append(int(k))
      except ValueError:
        pass
    pos = list(set(pos))
    pos.sort()
    return [d[str(i)] for i in pos]

def validate_params(method, D):
  if type(D['params']) == Object:
    keys = method.json_arg_types.keys()
    if len(keys) != len(D['params']):
      raise InvalidParamsError('Not eough params provided for %s' % method.json_sig)
    for k in keys:
      if not k in D['params']:
        raise InvalidParamsError('%s is not a valid parameter for %s' 
                                 % (k, method.json_sig))
      if not Any.kind(D['params'][k]) == method.json_arg_types[k]:
        raise InvalidParamsError('%s is not the correct type %s for %s'
          % (type(D['params'][k]), method.json_arg_types[k], method.json_sig))
  elif type(D['params']) == Array:
    arg_types = method.json_arg_types.values()
    try:
      for i, arg in enumerate(D['params']):
        if not Any.kind(arg) == arg_types[i]:
          raise InvalidParamsError('%s is not the correct type %s for %s'
            % (type(arg), arg_types[i], method.json_sig))
    except IndexError:
      raise InvalidParamsError('Too many params provided for %s' % method.json_sig)
    else:
      if len(D['params']) != len(arg_types):
        raise InvalidParamsError('Not enouh params provided for %s' % method.json_sig)


class JSONRPCSite(object):
  "A JSON-RPC Site"
  def __init__(self):
    self.urls = {}
    self.uuid = str(uuid1())
    self.version = '1.0'
    self.name = 'django-json-rpc'
    self.register('system.describe', self.describe)
  
  def register(self, name, method):
    self.urls[unicode(name)] = method
  
  def empty_response(self, version='1.0'):
    resp = {'id': None}
    if version == '1.1':
      resp['version'] = version
      return resp
    if version == '2.0':
      resp['jsonrpc'] = version
    resp.update({'error': None, 'result': None})
    return resp
  
  def validate_get(self, request, method):
    encode_get_params = lambda r: dict([(k, v[0] if len(v) == 1 else v)
                                         for k, v in r])
    if request.method == 'GET':
      method = unicode(method)
      if method in self.urls and getattr(self.urls[method], 'json_safe', False):
        D = {
          'params': encode_get_params(request.GET.lists()),
          'method': method,
          'id': 'jsonrpc',
          'version': '1.1'
        }
        return True, D
    return False, {}
  
  def response_dict(self, request, D, is_batch=False, version_hint='1.0'):
    version = version_hint
    response = self.empty_response(version=version)
    apply_version = {'2.0': lambda f, r, p: f(r, **encode_kw(p)) if type(p) is dict else f(r, *p),
                     '1.1': lambda f, r, p: f(r, *encode_arg11(p), **encode_kw(encode_kw11(p))),
                     '1.0': lambda f, r, p: f(r, *p)}
    
    try:
      if 'method' not in D or 'params' not in D:
        raise InvalidParamsError('Request requires str:"method" and list:"params"')
      if D['method'] not in self.urls:
        raise MethodNotFoundError('Method not found. Available methods: %s' % (
                        '\n'.join(self.urls.keys())))
      
      if 'jsonrpc' in D:
        if str(D['jsonrpc']) not in apply_version:
          raise InvalidRequestError('JSON-RPC version %s not supported.' % D['jsonrpc'])
        version = request.jsonrpc_version = response['jsonrpc'] = str(D['jsonrpc'])
      elif 'version' in D:
        if str(D['version']) not in apply_version:
          raise InvalidRequestError('JSON-RPC version %s not supported.' % D['version'])
        version = request.jsonrpc_version = response['version'] = str(D['version'])
      else:
        request.jsonrpc_version = '1.0'
        
      method = self.urls[str(D['method'])]
      if getattr(method, 'json_validate', False):
        validate_params(method, D)
      R = apply_version[version](method, request, D['params'])
      
      assert sum(map(lambda e: isinstance(R, e), 
        (dict, str, unicode, int, long, list, set, NoneType, bool))), \
        "Return type not supported"
      
      if 'id' in D and D['id'] is not None: # regular request
        response['result'] = R
        response['id'] = D['id']
        if version == '1.1' and 'error' in response:
          response.pop('error')
      elif is_batch: # notification, not ok in a batch format, but happened anyway
        raise InvalidRequestError
      else: # notification
        return None, 204
      
      status = 200
    
    except Error, e:
      response['error'] = e.json_rpc_format
      if version == '1.1' and 'result' in response:
        response.pop('result')
      status = e.status    
    except Exception, e:
      # exception missed by others
      other_error = OtherError(e)
      response['error'] = other_error.json_rpc_format
      status = other_error.status    
      if version == '1.1' and 'result' in response:
        response.pop('result')
    
    return response, status
  
  @csrf_exempt
  def dispatch(self, request, method=''):      
    from django.http import HttpResponse
    from django.core.serializers.json import DjangoJSONEncoder
    
    try:
      # in case we do something json doesn't like, we always get back valid json-rpc response
      response = self.empty_response()
      if request.method.lower() == 'get':
        valid, D = self.validate_get(request, method)
        if not valid:
          raise InvalidRequestError('The method you are trying to access is '
                                    'not availble by GET requests')
      elif not request.method.lower() == 'post':
        raise RequestPostError
      else:
        try:
          D = loads(request.raw_post_data)
        except:
          raise InvalidRequestError
      
      if type(D) is list:
        response = [self.response_dict(request, d, is_batch=True)[0] for d in D]
        status = 200
      else:
        response, status = self.response_dict(request, D)
        if response is None and (not u'id' in D or D[u'id'] is None): # a notification
          return HttpResponse('', status=status)
      
      json_rpc = dumps(response, cls=DjangoJSONEncoder)
    except Error, e:
      response['error'] = e.json_rpc_format
      status = e.status
      json_rpc = dumps(response, cls=DjangoJSONEncoder)
    except Exception, e:
      # exception missed by others
      other_error = OtherError(e)
      response['result'] = None
      response['error'] = other_error.json_rpc_format
      status = other_error.status    
      
      json_rpc = dumps(response,cls=DjangoJSONEncoder)
    
    return HttpResponse(json_rpc, status=status, content_type='application/json-rpc')
  
  def procedure_desc(self, key):
    M = self.urls[key]
    return {
      'name': M.json_method,
      'summary': M.__doc__,
      'idempotent': M.json_safe,
      'params': [{'type': str(Any.kind(t)), 'name': k} 
        for k, t in M.json_arg_types.iteritems()],
      'return': {'type': M.json_return_type}}
  
  def service_desc(self):
    return {
      'sdversion': '1.0',
      'name': self.name,
      'id': 'urn:uuid:%s' % str(self.uuid),
      'summary': self.__doc__,
      'version': self.version,
      'procs': [self.procedure_desc(k) 
        for k in self.urls.iterkeys()
          if self.urls[k] != self.describe]}
  
  def describe(self, request):
    return self.service_desc()


jsonrpc_site = JSONRPCSite()



def _types_gen(T):
  yield T
  if hasattr(T, 't'):
    for l in T.t:
      yield l
      if hasattr(l, 't'):
        for ll in _types_gen(l):
          yield ll


class Type(type):
  """ A rudimentary extension to `type` that provides polymorphic
  types for run-time type checking of JSON data types. IE:
  
  assert type(u'') == String
  assert type('') == String
  assert type('') == Any
  assert Any.kind('') == String
  assert Any.decode('str') == String
  assert Any.kind({}) == Object
  """
  
  def __init__(self, *args, **kwargs):
    type.__init__(self, *args, **kwargs)
  
  def __eq__(self, other):
    for T in _types_gen(self):
      if isinstance(other, Type):
        if T in other.t:
          return True
      if type.__eq__(T, other):
        return True
    return False
  
  def __str__(self):
    return getattr(self, '_name', 'unknown')
  
  def N(self, n):
    self._name = n
    return self
  
  def I(self, *args):
    self.t = list(args)
    return self
  
  def kind(self, t):
    if type(t) is Type:
      return t
    ty = lambda t: type(t)
    if type(t) is type:
      ty = lambda t: t
    return reduce(
      lambda L, R: R if (hasattr(R, 't') and ty(t) == R) else L,
      filter(lambda T: T is not Any, 
        _types_gen(self)))
  
  def decode(self, n):
    return reduce(
      lambda L, R: R if (str(R) == n) else L,
      _types_gen(self))

# JSON primatives and data types
Object = Type('Object', (object,), {}).I(dict).N('obj')
Number = Type('Number', (object,), {}).I(int, long).N('num')
Boolean = Type('Boolean', (object,), {}).I(bool).N('bit')
String = Type('String', (object,), {}).I(str, unicode).N('str')
Array = Type('Array', (object,), {}).I(list, set, tuple).N('arr')
Nil = Type('Nil', (object,), {}).I(type(None)).N('nil')
Any = Type('Any', (object,), {}).I(
        Object, Number, Boolean, String, Array, Nil).N('any')



import urllib
import uuid
from jsonrpc._json import loads, dumps
from jsonrpc.types import *
        
class ServiceProxy(object):
  def __init__(self, service_url, service_name=None, version='1.0'):
    self.__version = str(version)
    self.__service_url = service_url
    self.__service_name = service_name

  def __getattr__(self, name):
    if self.__service_name != None:
      name = "%s.%s" % (self.__service_name, name)
    return ServiceProxy(self.__service_url, name, self.__version)
  
  def __repr__(self):
    return {"jsonrpc": self.__version,
            "method": self.__service_name}
  
  def __call__(self, *args, **kwargs):
    params = kwargs if len(kwargs) else args
    if Any.kind(params) == Object and self.__version != '2.0':
      raise Exception('Unsupport arg type for JSON-RPC 1.0 '
                     '(the default version for this client, '
                     'pass version="2.0" to use keyword arguments)')
    r = urllib.urlopen(self.__service_url,
                        dumps({
                          "jsonrpc": self.__version,
                          "method": self.__service_name,
                          'params': params,
                          'id': str(uuid.uuid1())})).read()
    return loads(r)



try:
  from django.utils.translation import gettext as _
  _("You're lazy...") # this function lazy-loads settings
except (ImportError, NameError):
  _ = lambda t, *a, **k: t

class Error(Exception):
  """ Error class based on the JSON-RPC 2.0 specs 
      http://groups.google.com/group/json-rpc/web/json-rpc-1-2-proposal 
      
      code    - number
      message - string
      data    - object
      
      status  - number    from http://groups.google.com/group/json-rpc/web/json-rpc-over-http JSON-RPC over HTTP Errors section
  """
  
  code = 0
  message = None
  data = None
  status = 500
  
  def __init__(self, message=None):
    """ Setup the Exception and overwrite the default message """
    if message is not None:
      self.message = message
  
  @property
  def json_rpc_format(self):
    """ return the Exception data in a format for JSON-RPC """
    
    error = {
        'name': str(self.__class__.__name__),
        'code': self.code,
        'message': "%s: %s" % (str(self.__class__.__name__), str(self.message)),
        'data': self.data}

    from django.conf import settings
    
    if settings.DEBUG:
        import sys, traceback
        error['stack'] = traceback.format_exc()
        error['executable'] = sys.executable

    return error

# Exceptions
# from http://groups.google.com/group/json-rpc/web/json-rpc-1-2-proposal

# The error-codes -32768 .. -32000 (inclusive) are reserved for pre-defined errors. 
# Any error-code within this range not defined explicitly below is reserved for future use

class ParseError(Error):
  """ Invalid JSON. An error occurred on the server while parsing the JSON text. """
  code = -32700
  message = _('Parse error.')
  
class InvalidRequestError(Error):
  """ The received JSON is not a valid JSON-RPC Request. """
  code = -32600
  message = _('Invalid Request.')
  status = 400
  
class MethodNotFoundError(Error):
  """ The requested remote-procedure does not exist / is not available. """
  code = -32601
  message = _('Method not found.')
  status = 404
  
class InvalidParamsError(Error):
  """ Invalid method parameters. """
  code = -32602
  message = _('Invalid params.')
  
class ServerError(Error):
  """ Internal JSON-RPC error. """
  code = -32603	
  message = _('Internal error.')
  
# -32099..-32000    Server error.     Reserved for implementation-defined server-errors.  


# The remainder of the space is available for application defined errors.

class RequestPostError(InvalidRequestError):
  """ JSON-RPC requests must be POST """
  message = _('JSON-RPC requests must be POST')

class InvalidCredentialsError(Error):
  """ Invalid login credentials """
  code = 401
  message = _('Invalid login credentials')
  status = 401
  
class OtherError(Error):
  """ catchall error """
  code = 500
  message = _('Error missed by other execeptions')
  status = 500



#!/usr/bin/env python

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
  name="django-json-rpc",
  version="0.6.0",
  description="A simple JSON-RPC implementation for Django",
  long_description="""
Features:
    * Simple, pythonic API
    * Support for Django authentication
    * Supports JSON-RPC 1.0, 1.1, 1.2 and 2.0 Spec
    * Proxy to test your JSON Service
    * Run-time type checking
    * Graphical JSON-RPC browser and web console
    * Provides system.describe


**The basic API**

::

    ## myproj/myapp/views.py
    
    from jsonrpc import jsonrpc_method
    
    @jsonrpc_method('myapp.sayHello')
    def whats_the_time(request, name='Lester'):
      return "Hello %s" % name
    
    @jsonrpc_method('myapp.gimmeThat', authenticated=True)
    def something_special(request, secret_data):
      return {'sauce': ['authenticated', 'sauce']}
    
    
    ## myproj/urls.py
    
    from django.conf.urls.defaults import *
    from jsonrpc import jsonrpc_site
    import myproj.myapp.views # you must import the views that need connected
    
    urlpatterns += patterns('', 
      url(r'^json/', jsonrpc_site.dispatch, name="jsonrpc_mountpoint"),
      url(r'^json/browse/', 'jsonrpc.views.browse', name="jsonrpc_browse"), # for the graphical browser/web console only, omissible
      (r'^json/(?P<method>[a-zA-Z0-9.]+)$', jsonrpc_site.dispatch) # for HTTP GET only, also omissible
    )


**To test your service**

You can test your service using the provided graphical browser and console,
available at http://YOUR_URL/json/browse/ (if using the url patterns from above)
or with the included ServiceProxy::

    >>> from jsonrpc.proxy import ServiceProxy

    >>> s = ServiceProxy('http://localhost:8080/json/')

    >>> s.myapp.sayHello('Sam')
    {u'error': None, u'id': u'jsonrpc', u'result': u'Hello Sam'}

    >>> s.myapp.gimmeThat('username', 'password', 'test data')
    {u'error': None, u'id': u'jsonrpc', u'result': {u'sauce': [u'authenticated', u'sauce']}}

Method Browser:

.. image:: http://samuraiblog.com/wordpress/wp-content/uploads/2009/11/jsonrpcbrowserscreen.png

We add the `jsonrpc_version` variable to the request object. It be either
'1.0', '1.1' or '2.0'. Arg.

Guide
=====

Adding JSON-RPC to your application
-----------------------------------

**1. Install django-json-rpc**

::

    git clone git://github.com/samuraisam/django-json-rpc.git
    cd django-json-rpc
    python setup.py install

    # Add 'jsonrpc' to your INSTALLED_APPS in your settings.py file

**2. Write JSON-RPC methods**

::

    from jsonrpc import jsonrpc_method

    @jsonrpc_method('app.register')
    def register_user(request, username, password):
      u = User.objects.create_user(username, 'internal@app.net', password)
      u.save()
      return u.__dict__

    @jsonrpc_method('app.change_password', authenticated=True)
    def change_password(request, new_password):
      request.user.set_password(new_password)
      request.user.save()
      return u.__dict__

**3. Add the JSON-RPC mountpoint and import your views**

::

    from jsonrpc import jsonrpc_site
    import app.views

    urlpatterns = patterns('', 
      url(r'^json/$', jsonrpc_site.dispatch, name='jsonrpc_mountpoint'),
      # ... among your other URLs
    )


The jsonrpc_method decorator
----------------------------
Wraps a function turns it into a json-rpc method. Adds several attributes to 
the function speific to the JSON-RPC machinery and adds it to the default 
jsonrpc_site if one isn't provided. You must import the module containing these
functions in your urls.py.

``jsonrpc.jsonrpc_method(name, authenticated=False, safe=False, validate=False)``

Arguments::

    name
      
        The name of your method. IE: `namespace.methodName` The method name
        can include type information, like `ns.method(String, Array) -> Nil`.

    authenticated=False   

        Adds `username` and `password` arguments to the beginning of your 
        method if the user hasn't already been authenticated. These will 
        be used to authenticate the user against `django.contrib.authenticate` 
        If you use HTTP auth or other authentication middleware, `username` 
        and `password` will not be added, and this method will only check 
        against `request.user.is_authenticated`.

        You may pass a callablle to replace `django.contrib.auth.authenticate`
        as the authentication method. It must return either a User or `None`
        and take the keyword arguments `username` and `password`.

    safe=False

        Designates whether or not your method may be accessed by HTTP GET. 
        By default this is turned off.
  
    validate=False

        Validates the arguments passed to your method based on type 
        information provided in the signature. Supply type information by 
        including types in your method declaration. Like so:

        @jsonrpc_method('myapp.specialSauce(Array, String)', validate=True)
        def special_sauce(self, ingredients, instructions):
          return SpecialSauce(ingredients, instructions)

        Calls to `myapp.specialSauce` will now check each arguments type
        before calling `special_sauce`, throwing an `InvalidParamsError` 
        when it encounters a discrepancy. This can significantly reduce the
        amount of code required to write JSON-RPC services.
  
    site=default_site
      
        Defines which site the jsonrpc method will be added to. Can be any 
        object that provides a `register(name, func)` method.


Using type checking on methods (Python 2.6 or greater)
------------------------------------------------------

When writing web services you often end up manually checking the 
types of parameters passed. django-json-rpc provides a way to eliminate 
much of that code by specifying the types in your method signature. As 
specified in the JSON-RPC spec the available types are ``Object Array Number 
Boolean String Nil`` and ``Any`` meaning any type::

      @jsonrpc_method('app.addStrings(arg1=String, arg2=String) -> String', validate=True)
      def add_strings(request, arg1, arg2):
        return arg1 + arg2

However contrived this example, a lot of extra information about our
function is available. The ``system.describe`` method will automatically
be able to provide more information about the parameters and return type.
Provide ``validate=True`` to the ``jsonrpc_method`` decorator and you can be
guaranteed to receive two string objects when ``add_strings`` is called.

**Note:** Return type information is used only for reference, return value
types are not checked.

Types can be specified a number of ways, the following are all equivalent::

      # using JSON types:
      @jsonrpc_method('app.findSelection(query=Object, limit=Number)')

      # using Python types:
      @jsonrpc_method('app.findSelection(query=dict, limit=int)')

      # with mixed keyword parameters
      @jsonrpc_method('app.findSelection(dict, limit=int)')

      # with no keyword parameters
      @jsonrpc_method('app.findSelection(dict, int)')

      # with a return value
      @jsonrpc_method('app.findSelection(dict, int) -> list')

Using the browser
-----------------

To access the browser simply add another entry to your ``urls.py`` file. Make 
sure to include the name attribute of each url::

    urlpatterns += patterns('', 
      url(r'^json/browse/$', 'jsonrpc.views.browse', name='jsonrpc_browser')
    )


Enabling HTTP-GET
-----------------

JSON-RPC 1.1 includes support for methods which are accessible by HTTP GET 
which it calls idempotent. Add the following to your ``urls.py`` file to set 
up the GET URL::

    urlpatterns += patterns('', 
      (r'^json/(?P<method>[a-zA-Z0-9.-_]+)$', jsonrpc_site.dispatch),
    )

Each method that you want to be accessible by HTTP GET must also be marked safe
in the method decorator::

    @jsonrpc_method('app.trimTails(String)', safe=True)
    def trim_tails(request, arg1):
      return arg1[:5]

You can then call the method by loading ``/jsonrpc/app.trimTails?arg1=omgnowai``

Using authentication on methods
-------------------------------

There is no specific support for authentication in the JSON-RPC spec beyond 
whatever authentication the transport offers. To restrict access to methods 
to registered users provide ``authenticated=True`` to the method decorator. Doing 
so will add two arguments to the beginning of your method signature, ``username`` 
and ``password`` (and always in that order). By default, the credentials are 
authenticated against the builtin ``User`` database but any method can be used::

    @jsonrpc_method('app.thupertheecrit', authenticated=True)
    def thupertheecrit(request, value):
      p = request.user.get_profile()
      p.theecrit = value
      p.save()
      return p.__dict__

Using your own authentication method::

    def mah_authenticate(username, password):
      return CustomUserClass.authenticate(username, password)

    @jsonrpc_method('app.thupertheecrit', authenticated=mah_authenticate)
    def thupertheecrit(request, value):
      request.user.theecrit = value
      request.user.save()
      return request.user.__dict__

In case authentication is handled before your method is called, like in some 
middleware, providing ``authenticated=True`` to the method decorator will only 
check that ``request.user`` is authenticated and won't add any parameters to 
the beginning of your method.

""",
  author="Samuel Sutch",
  author_email="samuraiblog@gmail.com",
  license="MIT",
  url="http://github.com/samuraisam/django-json-rpc/tree/master",
  download_url="http://github.com/samuraisam/django-json-rpc/tree/master",
  classifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Software Development :: Libraries :: Python Modules'],
  packages=['jsonrpc'],
  install_requires=['Django>=1.0'],
  package_data={'jsonrpc': ['templates/*']})


import re
from inspect import getargspec
from functools import wraps
from django.utils.datastructures import SortedDict
from jsonrpc.site import jsonrpc_site
from jsonrpc.types import *
from jsonrpc.exceptions import *

default_site = jsonrpc_site
KWARG_RE = re.compile(
  r'\s*(?P<arg_name>[a-zA-Z0-9_]+)\s*=\s*(?P<arg_type>[a-zA-Z]+)\s*$')
SIG_RE = re.compile(
  r'\s*(?P<method_name>[a-zA-Z0-9._]+)\s*(\((?P<args_sig>[^)].*)?\)'
  r'\s*(\->\s*(?P<return_sig>.*))?)?\s*$')

class JSONRPCTypeCheckingUnavailable(Exception): pass

def _type_checking_available(sig='', validate=False):
  if not hasattr(type, '__eq__') and validate: # and False:
    raise JSONRPCTypeCheckingUnavailable(
      'Type checking is not available in your version of Python '
      'which is only available in Python 2.6 or later. Use Python 2.6 '
      'or later or disable type checking in %s' % sig)

def _validate_arg(value, expected):
  "Returns whether or not ``value`` is the ``expected`` type."
  if type(value) == expected:
    return True
  return False

def _eval_arg_type(arg_type, T=Any, arg=None, sig=None):
  """
  Returns a type from a snippit of python source. Should normally be
  something just like 'str' or 'Object'.
  
    arg_type      the source to be evaluated
    T             the default type
    arg           context of where this type was extracted
    sig           context from where the arg was extracted
  
  Returns a type or a Type
  """
  try:
    T = eval(arg_type)
  except Exception, e:
    raise ValueError('The type of %s could not be evaluated in %s for %s: %s' %
                    (arg_type, arg, sig, str(e)))
  else:
    if type(T) not in (type, Type):
      raise TypeError('%s is not a valid type in %s for %s' %
                      (repr(T), arg, sig))
    return T

def _parse_sig(sig, arg_names, validate=False):
  """
  Parses signatures into a ``SortedDict`` of paramName => type.
  Numerically-indexed arguments that do not correspond to an argument
  name in python (ie: it takes a variable number of arguments) will be
  keyed as the stringified version of it's index.
  
    sig         the signature to be parsed
    arg_names   a list of argument names extracted from python source
  
  Returns a tuple of (method name, types dict, return type)
  """
  d = SIG_RE.match(sig)
  if not d:
    raise ValueError('Invalid method signature %s' % sig)
  d = d.groupdict()
  ret = [(n, Any) for n in arg_names]
  if 'args_sig' in d and type(d['args_sig']) is str and d['args_sig'].strip():
    for i, arg in enumerate(d['args_sig'].strip().split(',')):
      _type_checking_available(sig, validate)
      if '=' in arg:
        if not type(ret) is SortedDict:
          ret = SortedDict(ret)
        dk = KWARG_RE.match(arg)
        if not dk:
          raise ValueError('Could not parse arg type %s in %s' % (arg, sig))
        dk = dk.groupdict()
        if not sum([(k in dk and type(dk[k]) is str and bool(dk[k].strip()))
            for k in ('arg_name', 'arg_type')]):
          raise ValueError('Invalid kwarg value %s in %s' % (arg, sig))
        ret[dk['arg_name']] = _eval_arg_type(dk['arg_type'], None, arg, sig)
      else:
        if type(ret) is SortedDict:
          raise ValueError('Positional arguments must occur '
                           'before keyword arguments in %s' % sig)
        if len(ret) < i + 1:
          ret.append((str(i), _eval_arg_type(arg, None, arg, sig)))
        else:
          ret[i] = (ret[i][0], _eval_arg_type(arg, None, arg, sig))
  if not type(ret) is SortedDict:
    ret = SortedDict(ret)
  return (d['method_name'], 
          ret, 
          (_eval_arg_type(d['return_sig'], Any, 'return', sig)
            if d['return_sig'] else Any))

def _inject_args(sig, types):
  """
  A function to inject arguments manually into a method signature before
  it's been parsed. If using keyword arguments use 'kw=type' instead in
  the types array.
    
    sig     the string signature
    types   a list of types to be inserted
    
  Returns the altered signature.
  """
  if '(' in sig:
    parts = sig.split('(')
    sig = '%s(%s%s%s' % (
      parts[0], ', '.join(types), 
      (', ' if parts[1].index(')') > 0 else ''), parts[1]
    )
  else:
    sig = '%s(%s)' % (sig, ', '.join(types))
  return sig

def jsonrpc_method(name, authenticated=False, safe=False, validate=False,
                   site=default_site):
  """
  Wraps a function turns it into a json-rpc method. Adds several attributes
  to the function speific to the JSON-RPC machinery and adds it to the default
  jsonrpc_site if one isn't provided. You must import the module containing
  these functions in your urls.py.
  
    name
        
        The name of your method. IE: `namespace.methodName` The method name
        can include type information, like `ns.method(String, Array) -> Nil`.

    authenticated=False   

        Adds `username` and `password` arguments to the beginning of your 
        method if the user hasn't already been authenticated. These will 
        be used to authenticate the user against `django.contrib.authenticate` 
        If you use HTTP auth or other authentication middleware, `username` 
        and `password` will not be added, and this method will only check 
        against `request.user.is_authenticated`.

        You may pass a callablle to replace `django.contrib.auth.authenticate`
        as the authentication method. It must return either a User or `None`
        and take the keyword arguments `username` and `password`.

    safe=False

        Designates whether or not your method may be accessed by HTTP GET. 
        By default this is turned off.
    
    validate=False

        Validates the arguments passed to your method based on type 
        information provided in the signature. Supply type information by 
        including types in your method declaration. Like so:

        @jsonrpc_method('myapp.specialSauce(Array, String)', validate=True)
        def special_sauce(self, ingredients, instructions):
          return SpecialSauce(ingredients, instructions)

        Calls to `myapp.specialSauce` will now check each arguments type
        before calling `special_sauce`, throwing an `InvalidParamsError` 
        when it encounters a discrepancy. This can significantly reduce the
        amount of code required to write JSON-RPC services.
    
    site=default_site
        
        Defines which site the jsonrpc method will be added to. Can be any 
        object that provides a `register(name, func)` method.
    
  """
  def decorator(func):
    arg_names = getargspec(func)[0][1:]
    X = {'name': name, 'arg_names': arg_names}
    if authenticated:
      if authenticated is True:
        # TODO: this is an assumption
        X['arg_names'] = ['username', 'password'] + X['arg_names']
        X['name'] = _inject_args(X['name'], ('String', 'String'))
        from django.contrib.auth import authenticate
        from django.contrib.auth.models import User
      else:
        authenticate = authenticated
      @wraps(func)  
      def _func(request, *args, **kwargs):
        user = getattr(request, 'user', None)
        is_authenticated = getattr(user, 'is_authenticated', lambda: False)
        if ((user is not None 
              and callable(is_authenticated) and not is_authenticated()) 
            or user is None):
          user = None
          try:
            creds = args[:2]
            user = authenticate(username=creds[0], password=creds[1])
            if user is not None:
              args = args[2:]
          except IndexError: 
            if 'username' in kwargs and 'password' in kwargs:
              user = authenticate(username=kwargs['username'],
                                  password=kwargs['password'])
              if user is not None:
                kwargs.pop('username')
                kwargs.pop('password')
            else:
              raise InvalidParamsError(
                'Authenticated methods require at least '
                '[username, password] or {username: password:} arguments')
          if user is None:
            raise InvalidCredentialsError
          request.user = user
        return func(request, *args, **kwargs)
    else:
      _func = func
    method, arg_types, return_type = \
      _parse_sig(X['name'], X['arg_names'], validate)
    _func.json_args = X['arg_names']
    _func.json_arg_types = arg_types
    _func.json_return_type = return_type
    _func.json_method = method
    _func.json_safe = safe
    _func.json_sig = X['name']
    _func.json_validate = validate
    site.register(method, _func)
    return _func
  return decorator
